/*	allegroHighScore Class v.1.1	FEB 19 2001
	Justin Todd 
    Features: Add and view records, auto-sort, simple to use.     
	Extended Features of allegroHighScore Class:
		AllegHS acts as a graphical frontend for HighScore Class in Allegro.
	Requirements: A C++ compiler
*/

#define MAX_RECORDS			10							//Maximum amount of high scores. WARNING!!!! Delete the current scores data file before changing this, or it won't work!
#define NAME_LENGTH			5							//Maxium amount of characters for name. WARNING!!!! Delete the current scores data file before changing this, or it won't work!
#define EMPTY_RECORD_NAME	"-Empty-\0"					//This is the name that is used for an emtpy record
#define FILE_NAME			"scores.dat"				//Name of the binary file to save highscores

//allegroHighScore defines

#define TEXT_COLOR			3							//Color of text
#define QUALIFY_MESSAGE		"Congratulations! You've made it into the highscores!"
#define XPOS				1							//x position of the "Name: joe" part
#define YPOS				50							//y position of the "Name: joe" part


#include<allegro.h>
#include<iostream.h>
#include<string.h>
#include<stdio.h>										//fopen, fread, fclose, exit, sprintf


class HighScore{										//The HighScore class can add=remove records,sort highscores, find averages, write to files. and much more
private:
	FILE *DataFile;										//The binary file containing saved scores
	struct RecordStruct{								//A record to hold the name and score
		char Name[NAME_LENGTH];							//Character string to hold name
		int Score;
	};
	RecordStruct Record[MAX_RECORDS];
public:
	HighScore();										//Constructor
	~HighScore();										//Destructor

	int addRecord(char name[NAME_LENGTH], int score);	//adds a record in the apropriate spot based on its score size
	int viewRecordScore(int position);					//This method returns the score for the desired record
	char * viewRecordName(int position);				//This method returns the name for the desired record
	int findAverageScore();								//Finds the average score of all of the records
	void clearAllRecords();								//Resets the highscores
	int testRecord(int score);							//Finds out if the users score is high enough to enter the highscores
};

HighScore::HighScore(){									//the constructor
	int x;												//a counter
	for(x=0;x<MAX_RECORDS;x++){							//Loop which initializes the record structure
		Record[x].Score=0;								//Initializes the scores
		strcpy(Record[x].Name,EMPTY_RECORD_NAME);		//Initializes the strings by putting the "empty" message for a name
	}
	DataFile=fopen(FILE_NAME,"rb");						//Open file for reading
	if(DataFile){										//If the file exists, copy the saved highscores from the file into the Record structure
		fread(&Record,sizeof(struct RecordStruct),\
			MAX_RECORDS,DataFile);						//Read the file into the highscore structure
		fclose(DataFile);								//Close the file
	}
}

HighScore::~HighScore(){
	DataFile=fopen(FILE_NAME,"wb");						//Open the datafile for writing				
	fwrite(&Record,sizeof(struct RecordStruct),\
		MAX_RECORDS,DataFile);							//Write the updated highscores to file
	fclose(DataFile);									//Close the file
}

int HighScore::addRecord(char name[NAME_LENGTH], int score){	//This method adds a record (if it even qualifies) in the appropriate location (sorted from great to least)
	int x,y;													//x and y are simply integer counters
	for(x=0;x<MAX_RECORDS;x++){									//a loop to go through the entire highscore list
		if(score>Record[x].Score){								//If the score trying to get into the highscore list is greater than the current score in the list, (Record[x].Score) then...
			for(y=(MAX_RECORDS-1);y>x;y--){						//...begin a loop that will move the list down to make room for it.  Note: the last score in the list will be disregarded
				Record[y].Score=Record[y-1].Score;				//"Move this score down one"
				strcpy(Record[y].Name,Record[y-1].Name);		//"Move this name down one"
			}
			Record[x].Score=score;								//Now that we've made an empty spot for the new score, lets fill it up
			strcpy(Record[x].Name,name);						//Place the users name into the record
			return 1;											//If 1 is returned, it means that the record made the highscore list and was successfully entered
		}
	}
	return 0;													//If 0 is returned, it means that this record did not make the highscore list
}

int HighScore::viewRecordScore(int position){					//This method returns the score for the desired record
	return Record[position].Score;
}

char * HighScore::viewRecordName(int position){					//This method returns the name for the desired record
	return Record[position].Name;
}

int HighScore::findAverageScore(){			//This method finds the average score of the entire list
	int sum,numRecords,x;					//numRecords keeps a count of non-zero records
	sum=numRecords=x=0;						//Sets the integers to zero
	for(x=0;x<MAX_RECORDS;x++){				//Go through all records
		if(Record[x].Score>0){				//If record has no value, disregard it
			sum+=Record[x].Score;			//Add this score to the current sum of scores
			numRecords++;					//Incremt numRecords
		}
	}
	return sum/numRecords;					//Return the average score
}

void HighScore::clearAllRecords(){						//Resets the highscores
	int x;
	for(x=0;x<MAX_RECORDS;x++){							//for loop to loop through entire Record structure
		Record[x].Score=0;								//Deletes score
		strcpy(Record[x].Name,EMPTY_RECORD_NAME);		//Deletes name
	}
}

int HighScore::testRecord(int score){					//Tests to see if the given score is enough to get into the highscores
	int x;												//Counter
	for(x=0;x<MAX_RECORDS;x++)							//Loops through the highscores...
		if(score>Record[x].Score) return 1;				//and if the given score is bigger than any score currently in the highscores, then return 0
	return 0;											//score wasn't high enough
}

/////////////////Beginning of allegroHighScore class////////////////////////

class allegroHighScore: public HighScore{						//allegroHighScore class inherits HighScore
public:
	allegroHighScore(){}										//Constructor
	int allegroEnterRecord(int score);							//Prompts user to enter his/her name (if their score is high enough), then adds their record
	void allegroDisplayAllRecords();							//graphically displays all records.
};

int allegroHighScore::allegroEnterRecord(int score){			//Prompts user to enter his/her name (if their score is high enough), then adds their record
	int position=0;												//This keeps track of the cursor position
	char ch;													//keyboard characters are stored in this variable
	char string[NAME_LENGTH]="\0";								//string to hold the users name

	if(testRecord(score)==0) return 0;							//See if users score is good enough for the highscore table, if not, return 0
	clear(screen);												//clear the screen
	textout(screen,font,QUALIFY_MESSAGE,1,1,TEXT_COLOR);	
	textout(screen,font,"Name: ",XPOS,YPOS,TEXT_COLOR);
	while(!key[KEY_ESC]){
		ch=0;													//Clears the key buffer
		ch=readkey();											//Gets a key from the keyboard	
		//The following code processes the key
		if(key[KEY_ENTER]) break;								//If user hits the enter key, be sure the line feed isn't saved into the last element of the string
		if(key[KEY_BACKSPACE]){									//When the user hits the backspace...
			if(position>0) position--;							//If the cursor is inside the textfield then decrease the cursor position
			string[position]=' ';								//Imitate a backspace (clear the preceeding character)
			ch=0;												//Clear keyboard buffer
		}
		if((ch!=0)&&(position<NAME_LENGTH)){					//If a character was pressed, and the cursor position is within text boundries...
			string[position]=ch;								//Make the string position equal the key entered
			position++;											//increment the string position
		}
		string[position+1]='\0';								//Terminitate the string
		textout(screen,font,string,XPOS+50,YPOS,TEXT_COLOR);	//Display the string
	}
	addRecord(string,score);									//Add users record to highscore table
	return 1;
}

void allegroHighScore::allegroDisplayAllRecords(){				//Graphically displays all records.
	int y;																//A counter
	char tempString[20];												//a string for use with the textout routine
	clear(screen);														//clears the screen
	textout(screen,font,"HighScores",1,1,TEXT_COLOR);
	for(y=0;y<MAX_RECORDS;y++){											//Loops through the record
		textout(screen,font,viewRecordName(y),1,(y+2)*15,TEXT_COLOR);	//Prints the name
		sprintf(tempString,"%i",viewRecordScore(y));					//Copies the score into a string
		textout(screen,font,tempString,150,(y+2)*15,TEXT_COLOR);		//Prints the score
	}
}
